@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
-#       This program is free software; you can redistribute it and/or modify
-#       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
-#       (at your option) any later version.
+#    Copyright (C) 2012  Christian Ledermann
 #
-#       This program is distributed in the hope that it will be useful,
-#       but WITHOUT ANY WARRANTY; without even the implied warranty of
-#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#       GNU General Public License for more details.
+#    This library is free software; you can redistribute it and/or
+#    modify it under the terms of the GNU Lesser General Public
+#    License as published by the Free Software Foundation; either
+#    version 2.1 of the License, or (at your option) any later version.
 #
-#       You should have received a copy of the GNU General Public License
-#       along with this program; if not, write to the Free Software
-#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#       MA 02110-1301, USA.
-
+#    This library is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+#    Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public
+#    License along with this library; if not, write to the Free Software
+#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 """
 KML 2.2 supports new elements for including data about the author and
 related website in your KML file. This information is displayed in geo
@@ -34,9 +34,9 @@ This library only  implements a subset of Atom that is useful with KML
 import logging
 logger = logging.getLogger('fastkml.atom')
 
-from config import etree
-from config import ATOMNS as NS
-from config import LXML
+from .config import etree
+from .config import ATOMNS as NS
+from .config import LXML
 
 import re
 regex = r"^[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+\.)*[a-zA-Z]{2,4}$"
@@ -143,9 +143,10 @@ class Link(object):
         """ Return the ATOM Object as serialized xml """
         if LXML and prettyprint:
             return etree.tostring(self.etree_element(), encoding='utf-8',
-                                    pretty_print=True)
+                                    pretty_print=True).decode('UTF-8')
         else:
-            return etree.tostring(self.etree_element(), encoding='utf-8')
+            return etree.tostring(self.etree_element(),
+                                    encoding='utf-8').decode('UTF-8')
 
 
 class _Person(object):
@@ -218,9 +219,10 @@ class _Person(object):
         """ Return the ATOM Object as serialized xml """
         if LXML and prettyprint:
             return etree.tostring(self.etree_element(), encoding='utf-8',
-                                    pretty_print=True)
+                                    pretty_print=True).decode('UTF-8')
         else:
-            return etree.tostring(self.etree_element(), encoding='utf-8')
+            return etree.tostring(self.etree_element(),
+                                    encoding='utf-8').decode('UTF-8')
 
 class Author(_Person):
     """ Names one author of the feed/entry. A feed/entry may have
