@@ -1,27 +1,31 @@
-# -*- coding: utf-8 -*-#       This program is free software; you can redistribute it and/or modify
-#       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
-#       (at your option) any later version.
+# -*- coding: utf-8 -*-
+#    Copyright (C) 2012  Christian Ledermann
 #
-#       This program is distributed in the hope that it will be useful,
-#       but WITHOUT ANY WARRANTY; without even the implied warranty of
-#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#       GNU General Public License for more details.
+#    This library is free software; you can redistribute it and/or
+#    modify it under the terms of the GNU Lesser General Public
+#    License as published by the Free Software Foundation; either
+#    version 2.1 of the License, or (at your option) any later version.
 #
-#       You should have received a copy of the GNU General Public License
-#       along with this program; if not, write to the Free Software
-#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#       MA 02110-1301, USA.
-
+#    This library is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+#    Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public
+#    License along with this library; if not, write to the Free Software
+#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 """ abstract base classes"""
 
-try:
-    from lxml import etree
-except ImportError:
-    import xml.etree.ElementTree as etree
 
-import config
+try:
+    import fastkml.config as config
+except  ImportError:
+    import config
+
+from .config import etree
+
+
 
 class _BaseObject(object):
     """ This is an abstract base class and cannot be used directly in a
@@ -70,6 +74,7 @@ class _BaseObject(object):
         """ Return the KML Object as serialized xml """
         if config.LXML and prettyprint:
             return etree.tostring(self.etree_element(), encoding='utf-8',
-                                    pretty_print=True)
+                                    pretty_print=True).decode('UTF-8')
         else:
-            return etree.tostring(self.etree_element(), encoding='utf-8')
+            return etree.tostring(self.etree_element(),
+                                    encoding='utf-8').decode('UTF-8')
